# Generated by Django 3.1.7 on 2021-11-16 13:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailcore", "0062_comment_models_and_pagesubscription"),
        ("wagtail_trash", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="trashcan",
            options={"verbose_name": "Trash Can", "verbose_name_plural": "Trash Cans"},
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="data",
            field=models.TextField(blank=True, verbose_name="Data"),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="page",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailcore.page",
                verbose_name="Page",
            ),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="parent",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.page",
                verbose_name="Parent",
            ),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="time_recycled",
            field=models.DateTimeField(auto_now_add=True, verbose_name="Time Recycled"),
        ),
        migrations.AlterField(
            model_name="trashcan",
            name="user",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to=settings.AUTH_USER_MODEL,
                verbose_name="User",
            ),
        ),
    ]



from wagtail_trash.models import TrashCan


class SkipSitemapIfInTrashMixin:
    def get_sitemap_urls(self, request=None):
        if self.in_trash_can():
            return []

        return super().get_sitemap_urls(request=request)

    def in_trash_can(self) -> bool:
        return TrashCan.objects.filter(page=self).exists()



import json

from django.test import TestCase
from wagtail.models import Page
from wagtail.test.utils import WagtailTestUtils

from wagtail_trash.utils import generate_page_data


class TestUtils(TestCase, WagtailTestUtils):
    def test_generate_page_data(self):
        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        sub_page = Page(title="1p 2p", has_unpublished_changes=False, live=True)
        top.add_child(instance=sub_page)
        sub_page_id = sub_page.id

        sub_sub_page = Page(title="1p 2p 3u", has_unpublished_changes=True, live=False)
        sub_page.add_child(instance=sub_sub_page)

        self.assertEqual(top.get_descendants().live().count(), 1)
        self.assertEqual(top.get_descendants().not_live().count(), 2)

        self.assertEqual(
            json.loads(generate_page_data(top)), {"published": [top.id, sub_page_id]}
        )



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.shortcuts import reverse
from django.test import TestCase
from wagtail.models import GroupPagePermission, Page
from wagtail.test.utils import WagtailTestUtils

from wagtail_trash.models import TrashCan, TrashCanPage
from wagtail_trash.views import trash_delete
from wagtail_trash.wagtail_hooks import TrashCanModelAdmin

trash_admin_url_helper = TrashCanModelAdmin().url_helper


class TestPermissions(TestCase, WagtailTestUtils):
    pass
    # def setUp(self):
    #     self.group = Group.objects.create(name="Wowie Kazowie")
    #     moderators = Group.objects.filter(name="Moderators").first()

    #     self.test_user_with_permission = get_user_model().objects.create_user(
    #         username="wowie",
    #         email="kazowie@wowoie.test",
    #         password="password123",
    #         first_name="Wowie",
    #         last_name="Kazowie",
    #     )

    #     self.test_user_without_permission = get_user_model().objects.create_user(
    #         username="not_worthy",
    #         email="not@worthy.test",
    #         password="hunter2",
    #         first_name="Not",
    #         last_name="Worthy",
    #     )

    #     admin_permission = Permission.objects.get(content_type__app_label='wagtailadmin', codename='access_admin')
    #     trash_permissions = Permission.objects.filter(content_type__app_label__startswith='wagtail_trash')

    #     self.test_user_with_permission.groups.add(self.group, moderators)
    #     self.test_user_with_permission.user_permissions.add(admin_permission, *trash_permissions)
    #     self.test_user_without_permission.groups.add(moderators)
    #     self.test_user_without_permission.user_permissions.add(admin_permission, *trash_permissions)

    #     self.root_page = Page.objects.get(url_path="/")

    #     self.top_page = Page(title="top_page")
    #     self.root_page.add_child(instance=self.top_page)

    #     self.sub_page = Page(title="top_page")
    #     self.top_page.add_child(instance=self.sub_page)

    #     self.group_permission = GroupPagePermission.objects.create(
    #         page=self.top_page, permission_type="edit", group=self.group
    #     )

    # def login_permissions(self):
    #     self.client.login(username="wowie", password="password123")

    # def login_no_permissions(self):
    #     self.client.login(username="not_worthy", password="hunter2")

    # def test_permissions(self):
    #     index_url = trash_admin_url_helper.get_action_url("index")

    #     self.login_permissions()

    #     resp = self.client.get(index_url)
    #     html = str(resp.content)

    #     self.assertEqual(html.count("data-object-pk"), 0)

    #     with self.register_hook("before_delete_page", trash_delete):
    #         delete_url = reverse("wagtailadmin_pages:delete", args=(self.top_page.id,))
    #         self.client.post(delete_url)

    #     resp = self.client.get(index_url)
    #     html = str(resp.content)
    #     print(html)

    #     self.assertEqual(html.count("data-object-pk"), 1)



from django.shortcuts import reverse
from django.test import TestCase
from wagtail.models import Page
from wagtail.test.utils import WagtailTestUtils

from tests.app.models import TestPage
from wagtail_trash.models import TrashCan, TrashCanPage
from wagtail_trash.views import trash_bulk_delete, trash_delete
from wagtail_trash.wagtail_hooks import TrashCanModelAdmin

trash_admin_url_helper = TrashCanModelAdmin().url_helper


class TestAdmin(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

    def test_visiting_bin_creates_one_if_not_exists(self):
        index_url = trash_admin_url_helper.get_action_url("index")

        assert TrashCanPage.objects.count() == 0

        self.client.get(index_url)

        assert TrashCanPage.objects.count() == 1

        self.client.get(index_url)

    def test_buttons(self):
        root_page = Page.objects.get(url_path="/")

        top_page = Page(title="top_page")
        root_page.add_child(instance=top_page)

        sub_page = Page(title="sub_page")
        top_page.add_child(instance=sub_page)

        sub_sub_page = Page(title="sub_sub_page")
        sub_page.add_child(instance=sub_sub_page)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(sub_sub_page.id,))
            self.client.post(delete_url)

            delete_url = reverse("wagtailadmin_pages:delete", args=(top_page.id,))
            self.client.post(delete_url)

        index = trash_admin_url_helper.get_action_url("index")
        resp = self.client.get(index)

        # Since sub_sub_pages parent is now in the trash can we should not show the "Restore"-button.
        self.assertFalse(
            reverse("wagtail_trash_restore", args=(sub_sub_page.id,))
            in str(resp.content)
        )
        self.assertTrue(
            reverse("wagtail_trash_move", args=(sub_sub_page.id,)) in str(resp.content)
        )

        # However - top page still has a parent (root) and should be able to be restored.
        self.assertTrue(
            reverse("wagtail_trash_restore", args=(top_page.id,)) in str(resp.content)
        )
        self.assertTrue(
            reverse("wagtail_trash_move", args=(top_page.id,)) in str(resp.content)
        )

    def test_removing_page_sends_it_to_trash_can(self):
        root_page = Page.objects.get(url_path="/")

        new_page = Page(title="new page")
        root_page.add_child(instance=new_page)

        assert TrashCan.objects.count() == 0

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(new_page.id,))
            self.client.post(delete_url)

        new_page = Page.objects.filter(title="new page")

        assert new_page.exists()
        assert new_page.child_of(TrashCanPage.objects.first())
        assert TrashCan.objects.count() == 1

    def test_trash_can_and_children_are_not_indexed_in_sitemap(self):
        root_page = Page.objects.get(url_path="/")

        new_page = TestPage(title="new page")
        root_page.add_child(instance=new_page)

        assert len(new_page.get_sitemap_urls()) != 0

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(new_page.id,))
            self.client.post(delete_url)

        trash_can_page = TrashCanPage.objects.first()
        assert len(trash_can_page.get_sitemap_urls()) == 0
        assert len(new_page.get_sitemap_urls()) == 0

    def test_bulk_delete_sets_and_unsets_slug(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/")

        new_page1 = Page(title="new page 1")

        p1 = root_page.add_child(instance=new_page1)

        p1_slug = p1.slug

        with self.register_hook("before_bulk_action", trash_bulk_delete):
            delete_url = reverse(
                "wagtail_bulk_action", args=("wagtailcore", "page", "delete")
            )
            id_query = "?id=" + "&id=".join([str(p1.id)])
            self.client.post(delete_url + id_query)

        p1.refresh_from_db()
        self.assertEqual(f"trash-{p1.pk}-{p1_slug}", p1.slug)

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse("wagtail_trash_restore", args=(p1.pk,))
            self.client.get(restore_url)

        p1.refresh_from_db()
        self.assertEqual(p1_slug, p1.slug)

    def test_bulk_delete_sends_to_trash_can(self):
        root_page = Page.objects.get(url_path="/")

        new_page1 = Page(title="new page 1")
        new_page2 = Page(title="new page 2")
        new_page3 = Page(title="new page 3")
        new_page4 = Page(title="new page 4")

        p1 = root_page.add_child(instance=new_page1)
        p2 = root_page.add_child(instance=new_page2)
        p3 = root_page.add_child(instance=new_page3)
        p4 = root_page.add_child(instance=new_page4)

        assert TrashCan.objects.count() == 0

        with self.register_hook("before_bulk_action", trash_bulk_delete):
            delete_url = reverse(
                "wagtail_bulk_action", args=("wagtailcore", "page", "delete")
            )
            id_query = "?id=" + "&id=".join(
                [str(p1.id), str(p2.id), str(p3.id), str(p4.id)]
            )
            self.client.post(delete_url + id_query)

        assert TrashCan.objects.count() == 4

    def test_removing_page_from_bin_deletes_it(self):
        root_page = Page.objects.get(url_path="/")

        new_page = Page(title="delete page")
        root_page.add_child(instance=new_page)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(new_page.id,))
            self.client.post(delete_url)
            assert TrashCan.objects.count() == 1

            trash_can_item = TrashCan.objects.first()
            delete_url = trash_admin_url_helper.get_action_url(
                "delete",
                trash_can_item.id,
            )
            self.client.post(delete_url)
            assert not Page.objects.filter(title="delete page")
            assert TrashCan.objects.count() == 0

    def test_removing_two_pages_with_same_slug(self):
        root_page = Page.objects.get(url_path="/")

        cat = Page(title="category 1", slug="category-1")
        cat_instance = root_page.add_child(instance=cat)
        cat_2 = Page(title="category 2", slug="category-2")
        cat_instance_2 = root_page.add_child(instance=cat_2)

        delete = Page(title="delete", slug="delete")
        cat_instance.add_child(instance=delete)
        delete_2 = Page(title="delete", slug="delete")
        cat_instance_2.add_child(instance=delete_2)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(delete.id,))
            self.client.post(delete_url)
            delete_url = reverse("wagtailadmin_pages:delete", args=(delete_2.id,))
            self.client.post(delete_url)

            self.assertEqual(TrashCan.objects.count(), 2)

    def test_removing_page_unpublishes_all_sub_pages(self):
        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        sub_page = Page(title="1p 2p", has_unpublished_changes=False, live=True)
        top.add_child(instance=sub_page)

        sub_sub_page = Page(title="1p 2p 3u", has_unpublished_changes=True, live=False)
        sub_page.add_child(instance=sub_sub_page)

        self.assertEqual(top.get_descendants(inclusive=True).live().count(), 2)
        self.assertEqual(top.get_descendants(inclusive=True).not_live().count(), 2)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(top.id,))
            self.client.post(delete_url)

        top.refresh_from_db()

        self.assertEqual(top.get_descendants(inclusive=True).live().count(), 0)
        self.assertEqual(top.get_descendants(inclusive=True).not_live().count(), 4)

    def test_restoring_page_re_publishes(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        sub_page = Page(title="1p 2p", has_unpublished_changes=False, live=True)
        top.add_child(instance=sub_page)

        sub_sub_page = Page(title="1p 2p 3u", has_unpublished_changes=True, live=False)
        sub_page.add_child(instance=sub_sub_page)

        self.assertEqual(top.get_descendants(inclusive=True).live().count(), 2)
        self.assertEqual(top.get_descendants(inclusive=True).not_live().count(), 2)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(top.id,))
            self.client.post(delete_url)

        top.refresh_from_db()

        self.assertEqual(top.get_descendants(inclusive=True).live().count(), 0)
        self.assertEqual(top.get_descendants(inclusive=True).not_live().count(), 4)

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse("wagtail_trash_restore", args=(top.id,))
            self.client.get(restore_url)

        top.refresh_from_db()

        self.assertEqual(top.get_descendants(inclusive=True).live().count(), 2)
        self.assertEqual(top.get_descendants(inclusive=True).not_live().count(), 2)
        self.assertEqual(TrashCan.objects.count(), 0)
        self.assertEqual(TrashCanPage.objects.first().get_children().count(), 0)

    def test_restoring_page_preserves_old_url(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/home/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        sub_page = Page(title="1p 2p", has_unpublished_changes=False, live=True)
        top.add_child(instance=sub_page)

        original_top_url = top.url_path

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(top.id,))
            self.client.post(delete_url)

        top.refresh_from_db()

        self.assertNotEqual(original_top_url, top.url_path)

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse("wagtail_trash_restore", args=(top.id,))
            self.client.get(restore_url)

        top.refresh_from_db()

        self.assertEqual(original_top_url, top.url_path)

    def test_restoring_page_custom_move_to(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(sub_page.id,))
            self.client.post(delete_url)

        sub_page.refresh_from_db()

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse(
                "wagtail_trash_move",
                args=(sub_page.id,),
            )
            self.client.post(restore_url, {"move_page": str(root_page.id)})

        sub_page.refresh_from_db()

        self.assertIn(
            sub_page.id,
            list(root_page.get_children().values_list("id", flat=True)),
        )

        self.assertEqual(TrashCan.objects.count(), 0)

    def test_restoring_page_custom_move_to_without_id_returns_validation_error(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        with self.register_hook("before_delete_page", trash_delete):
            delete_url = reverse("wagtailadmin_pages:delete", args=(sub_page.id,))
            self.client.post(delete_url)

        sub_page.refresh_from_db()

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse(
                "wagtail_trash_move",
                args=(sub_page.id,),
            )
            response = self.client.post(restore_url, {"move_page": ""})
            self.assertIn("This field is required", str(response.content))

    def test_move_view_renders(self):
        from wagtail_trash.wagtail_hooks import urlconf_time

        root_page = Page.objects.get(url_path="/")

        top = Page(title="1p", has_unpublished_changes=False, live=True)
        root_page.add_child(instance=top)

        sub_page = Page(title="1p 1u", has_unpublished_changes=True, live=False)
        top.add_child(instance=sub_page)

        with self.register_hook("register_admin_urls", urlconf_time):
            restore_url = reverse(
                "wagtail_trash_move",
                args=(sub_page.id,),
            )
            self.client.get(restore_url)



# Generated by Django 3.1.2 on 2020-10-29 13:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0052_pagelogentry"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="TrashCanPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
        migrations.CreateModel(
            name="TrashCan",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("time_recycled", models.DateTimeField(auto_now_add=True)),
                ("data", models.TextField(blank=True)),
                (
                    "page",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from datetime import timedelta

from django.core.management import call_command
from django.test import TestCase
from django.utils import timezone
from wagtail.models import Page
from wagtail.test.utils import WagtailTestUtils

from wagtail_trash.models import TrashCan, TrashCanPage


class TestEmptyTrashCommand(TestCase, WagtailTestUtils):
    def test_empty_bin_removes_pages_older_than_setting(self):
        root_page = Page.objects.get(url_path="/")

        new_page = Page(title="new page")
        root_page.add_child(instance=new_page)
        TrashCan.objects.create(page=new_page)

        old_page = Page(title="new page oldie")
        root_page.add_child(instance=old_page)
        TrashCan.objects.create(page=old_page)
        TrashCan.objects.filter(page__title="new page oldie").update(
            time_recycled=timezone.now() - timedelta(days=31)
        )

        self.assertEqual(TrashCan.objects.count(), 2)

        call_command("empty_trash", older_than_days=30)

        self.assertFalse(TrashCan.objects.filter(page_id=old_page.id).exists())
        self.assertTrue(TrashCan.objects.filter(page_id=new_page.id).exists())
        self.assertEqual(TrashCan.objects.count(), 1)


class TestDeleteStraypagesCommand(TestCase, WagtailTestUtils):
    def test_delete_stray_pages_removes_pages_from_trash_can_witout_trashcan_association(
        self,
    ):
        root_page = Page.objects.get(url_path="/")

        trash_can_page = TrashCanPage(title="Trash can")
        root_page.add_child(instance=trash_can_page)

        new_page = Page(title="New page")
        trash_can_page.add_child(instance=new_page)
        TrashCan.objects.create(page=new_page)

        stray_page = Page(title="Stray page")
        trash_can_page.add_child(instance=stray_page)

        self.assertEqual(trash_can_page.get_children().count(), 2)

        call_command("delete_stray_pages")

        self.assertEqual(trash_can_page.get_children().count(), 1)



from django.core.management.base import BaseCommand

from wagtail_trash.models import TrashCan, TrashCanPage


class Command(BaseCommand):
    """
    Delete stray pages from recycle bin(s)

    Example:
        ./manage.py delete_stray_pages
    """

    def handle(self, *args, **options):
        trash_can_pages = TrashCanPage.objects.all()
        num_deleted_pages = 0
        for page in trash_can_pages:
            for child_page in page.get_children():
                if TrashCan.objects.filter(page=child_page).exists():
                    continue

                child_page.delete()
                num_deleted_pages += 1

        self.stdout.write("Deleted {} stray pages".format(num_deleted_pages))
